﻿#region License Information
/* HeuristicLab
 * Copyright (C) 2002-2010 Heuristic and Evolutionary Algorithms Laboratory (HEAL)
 *
 * This file is part of HeuristicLab.
 *
 * HeuristicLab is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * HeuristicLab is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with HeuristicLab. If not, see <http://www.gnu.org/licenses/>.
 */
#endregion

using System;
using HeuristicLab.Common;
using HeuristicLab.Persistence.Default.CompositeSerializers.Storable;

namespace HeuristicLab.Services.Hive.Common {
  /// <summary>
  /// The MessageContainer is a container class for Messages. Its two parts are:
  /// the actual message itself and the JobId, refered by the message
  /// </summary>
  [StorableClass]
  [Serializable]
  public class MessageContainer : IDeepCloneable {

    public enum MessageType {
      FetchJob, 
      AbortJob, 
      JobAborted, 
      FinishedJob, 
      NoMessage, 
      SnapshotReady, 
      Shutdown, 
      JobFailed, 
      UptimeLimitDisconnect,
      UpdateCalendar, 
      AddChildJob,
      PauseJob,
      GetChildJobs,
      DeleteChildJobs,

      /// <summary>
      /// Slave should add itself
      /// </summary>
      AddSlaveInfo
    };

    [Storable]
    public MessageType Message { get; set; }
    [Storable]
    public Guid JobId { get; set; }

    public MessageContainer(MessageType message) {
      Message = message;
      JobId = Guid.Empty;
    }

    public MessageContainer(MessageType message, Guid jobId) {
      Message = message;
      JobId = jobId;
    }

    public virtual IDeepCloneable Clone(Cloner cloner) {
      MessageContainer clone = (MessageContainer)Activator.CreateInstance(this.GetType(), true);
      cloner.RegisterClonedObject(this, clone);
      clone.Message = this.Message;
      clone.JobId = this.JobId;
      return clone;
    }

    public object Clone() {
      return Clone(new Cloner());
    }
  }
}
