﻿#region License Information
/* HeuristicLab
 * Copyright (C) 2002-2010 Heuristic and Evolutionary Algorithms Laboratory (HEAL)
 *
 * This file is part of HeuristicLab.
 *
 * HeuristicLab is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * HeuristicLab is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with HeuristicLab. If not, see <http://www.gnu.org/licenses/>.
 */
#endregion

using System;
using System.Collections.Generic;
using System.IO;
using System.Linq;
using System.Xml.Serialization;
using HeuristicLab.Calendar;
using HeuristicLab.Hive.Contracts.BusinessObjects;
using HeuristicLab.Hive.Contracts.ResponseObjects;
using HeuristicLab.Hive.Tracing;

namespace HeuristicLab.Hive.Slave.Core.ConfigurationManager {
  public class UptimeManager {

    private AppointmentContainer appContainer = null;
    public AppointmentContainer AppContainer {
      get {
        if (appContainer == null)
          RestoreFromHDD();
        return appContainer;
      }
    }    
    public bool CalendarAvailable { get; set; }

    private static String path = System.IO.Directory.GetCurrentDirectory() + "\\plugins\\Hive.Slave.Jobs\\";

    private static UptimeManager instance = null;
    public static UptimeManager Instance {
      get {
        if (instance == null) {
          instance = new UptimeManager();
        }
        return instance;
      }
    }

    public UptimeManager() {
      RestoreFromHDD();
    }

    private void PersistToHDD() {
      XmlSerializer s = new XmlSerializer(typeof(AppointmentContainer));
      if (!Directory.Exists(path))
        Directory.CreateDirectory(path);
      TextWriter w = null;
      try {
        w = new StreamWriter(path + "calendar.xml");
        s.Serialize(w, AppContainer);
      }
      catch (Exception e) {
        Logger.Error("Persistance of the Calendar failed!", e);
      }
      finally {
        if (w != null)
          w.Close();
      }
    }

    private void RestoreFromHDD() {
      XmlSerializer s = new XmlSerializer(typeof(AppointmentContainer));
      if (File.Exists(Path.Combine(path, "calendar.xml"))) {
        TextReader r = null;

        try {
          r = new StreamReader(path + "calendar.xml");
          appContainer = (AppointmentContainer)s.Deserialize(r);
          CalendarAvailable = true;
        }
        catch (Exception e) {
          Logger.Error("Deserialization of Calendar failed", e);
          Logger.Info("Starting with a new one");
          appContainer = new AppointmentContainer();
          CalendarAvailable = false;
        }
        finally {
          if (r != null)
            r.Close();
        }
      } else {
        appContainer = new AppointmentContainer();
      }
    }

    public bool IsAllowedToCalculate() {
      return AppContainer.Appointments.Any(app => (DateTime.Now >= app.StartDate) && (DateTime.Now <= app.EndDate));
    }

    public bool SetAppointments(bool isLocal, bool isForced, IEnumerable<Appointment> appointments) {
      if (!isForced && !isLocal && AppContainer.IsLocal)
        return false;

      AppContainer.Appointments = new List<Appointment>(appointments);
      AppContainer.IsLocal = isLocal;
      AppContainer.Updated = DateTime.Now;
      CalendarAvailable = true;
     
      PersistToHDD();
      
      return true;
    }

    internal bool SetAppointments(bool isLocal, ResponseCalendar response) {
      IList<Appointment> app = new List<Appointment>();
      foreach (AppointmentDto appointmentDto in response.Appointments) {
        app.Add(new Appointment {
          AllDayEvent = appointmentDto.AllDayEvent,
          EndDate = appointmentDto.EndDate,
          StartDate = appointmentDto.StartDate,
          Recurring = appointmentDto.Recurring,

          RecurringId = appointmentDto.RecurringId,
          Locked = true,
          Subject = "Online",
        });
      }
      return SetAppointments(isLocal, response.ForceFetch, app);
    }
  }
}
